package jp.sourceforge.shovel.action.impl;

import static jp.sourceforge.shovel.ISessionConst.*;
import static jp.sourceforge.shovel.ViewType.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.seasar.framework.container.S2Container;

import jp.sourceforge.shovel.FormatType;
import jp.sourceforge.shovel.action.IFavoritesAction;
import jp.sourceforge.shovel.annotation.Perform;
import jp.sourceforge.shovel.entity.IFriendship;
import jp.sourceforge.shovel.entity.IListRefine;
import jp.sourceforge.shovel.entity.IStatus;
import jp.sourceforge.shovel.entity.IUser;
import jp.sourceforge.shovel.exception.ApplicationException;
import jp.sourceforge.shovel.form.IStatusesForm;
import jp.sourceforge.shovel.service.IDirectoryService;
import jp.sourceforge.shovel.service.IShovelService;

public class FavoritesActionImpl implements IFavoritesAction {
    IStatusesForm actionForm_;
    HttpServletRequest request_;
    HttpServletResponse response_;
    HttpSession session_;
    S2Container container_;
    
    @Perform(CSRF=false, login=false)
    public String perform() throws Exception {
        FormatType formatType = actionForm_.getFormatType();
        
        IShovelService shovelService = getShovelService();
        IDirectoryService directoryService = shovelService.getDirectoryService();
        
        String foreignKey = actionForm_.getForeignKey();
        
        IUser user;
        if (foreignKey == null || foreignKey.length() <= 0) {
            user = directoryService.getLoginUser();
        } else {
            user = directoryService.getUser(foreignKey);
        }
        
        int page = actionForm_.getPage();
        int limit = actionForm_.getCount();
        if (formatType.isHtml()) {
            IListRefine listRefine = (IListRefine)session_.getAttribute(S_LIST_REFINE);
            if (listRefine == null || !listRefine.getViewType().isFavorites()) {
                listRefine = (IListRefine)container_.getComponent(IListRefine.class);
                session_.setAttribute(S_LIST_REFINE, listRefine);
            } else if (actionForm_.getMethodType().isPage()) {
                listRefine.setPage(actionForm_.getPage());
            }
            listRefine.setViewType(FAVORITES);
            
            page = listRefine.getPage();
            limit = 0;
        }
        
        IStatus[] temp = shovelService.getFavorites(user.getForeignKey(), page, limit);
        IStatus[] statuses = shovelService.prepareForView(temp, limit, true);
        request_.setAttribute("statuses", statuses);
        
        String forward = null;
        switch (formatType) {
        case HTML:
            shovelService.prepareForView(new IUser[] {user}, 1);
            request_.setAttribute("user", user);
            IFriendship[] friendships = shovelService.getFriends(user.getForeignKey(), 0, 100);
            request_.setAttribute("viewAllFriends", friendships.length > 100);
            friendships = shovelService.prepareForView(friendships, 100, true);
            request_.setAttribute("friendships", friendships);
            
            boolean nextPage = false;
            if (statuses != null) {
                nextPage = temp.length > statuses.length;
            }
            request_.setAttribute("nextPage", nextPage);
            
            forward = actionForm_.isPartial() ? PARTIAL : FULL;
            break;
        case JSON:
        case XML:
            forward = formatType.getId();
            break;
        case ATOM:
        case RSS:
            shovelService.outputFeed(formatType, PUBLIC_TIMELINE, user, statuses);
            break;
        default:
            throw new ApplicationException("");
        }
        
        return forward;
    }
    public void setStatusesForm(IStatusesForm actionForm) {
        actionForm_ = actionForm;
    }
    public IShovelService getShovelService() {
        return (IShovelService)request_.getAttribute("shovelService");
    }
    public void setContainer(S2Container container) {
        container_ = container;
    }
    public void setRequest(HttpServletRequest request) {
        request_ = request;
    }
    public void setResponse(HttpServletResponse response) {
        response_ = response;
    }
    public void setSession(HttpSession session) {
        session_ = session;
    }
}
